#include "train.h"

static volatile unsigned char *_port = &PORTD;

Train::Train(const uint8_t address, uint8_t protocol):  _address(address), _protocol(protocol)
{
}

Train::Train()
{
    _address = 0;
}

void Train::setAddress(const uint8_t address)
{
    _address = address;
}

uint8_t Train::getAddress()
{
    return _address;
}

void Train::stop()
{
    lastDataPacket = 0;
    resendData();
}

bool Train::isActive()
{
    return lastDataPacket & 0b0000000111111111;
}


void Train::off()
{
    writePin(_port, _pinHighA, false);
    writePin(_port, _pinHighB, false);
    writePin(_port, _pinLowA,  true);
    writePin(_port, _pinLowB,  true);
}

void Train::setOutput(const uint8_t state)
{
    if(state == HIGH)
    {
        off();
        _delay_us(3);
        writePin(_port, _pinHighA, true);
        writePin(_port, _pinLowB,  false);
    }
    else if (state == LOW)
    {
        off();
        _delay_us(3);
        writePin(_port, _pinHighB, true);
        writePin(_port, _pinLowA,  false);
    }
    else
    {
        off();
    }
}

void Train::sendBit(const bool bit)
{
    if(bit)
    {
        setOutput(HIGH);
        _delay_us(170);
        setOutput(LOW);
        _delay_us(25);
    }
    else
    {
        setOutput(HIGH);
        _delay_us(20);
        setOutput(LOW);
        _delay_us(175);
    }
}

void Train::sendAddress()
{
    for(uint8_t i = 0; i < 8; i++) 
    {
        sendBit(_address & (1 << i));
    }
}

void Train::sendRaw(const uint16_t data)
{
    for(uint8_t j = 0; j < SEND_COUNT; j++)
    {
        sendAddress();
        for(uint8_t i = 0; i < 10; i++)
        {
            sendBit(data & (1 << i));
        }
        _delay_ms(2);
    }
}

void Train::setSpeed(uint8_t speed)
{
    if(speed != 0)speed = speed + 1;
    else if(speed > 15) speed = 15;
    for(uint8_t i = 0; i < 4; i++)
    {
                                 lastDataPacket = (lastDataPacket & ~(1 << (i+1)*2))     | (((uint16_t)speed & (1 << i)) << (i+1)*2-i);
        if(_protocol == M_DELTA) lastDataPacket = (lastDataPacket & ~(1 << ((i+1)*2+1))) | (((uint16_t)speed & (1 << i)) << ((i+1)*2+1-i));
    }
    resendData();
}

void Train::resendData()
{
    sendRaw(lastDataPacket);
}

uint16_t Train::getLastPacket()
{
    return lastDataPacket;
}

void Train::reverse()
{
    sendRaw(0b000001100);
    sendRaw(0b000001100);
}

void Train::sendFunction(const uint8_t function, bool enable)
{
    if(function == 0) lastDataPacket = (lastDataPacket & ~0b00000011) | (enable ? 0b00000011 : 0b00000000);
    else if(_protocol == M_DIGITAL && function <= 3)
    {
        lastDataPacket = lastDataPacket | 0b1000000000;
        if(enable) lastDataPacket = lastDataPacket |  (1 << (9-function*2));
        else       lastDataPacket = lastDataPacket & ~(1 << (9-function*2));
    }
    resendData();
}

void Train::setProtocol(const uint8_t protocol)
{
   _protocol = protocol;
}

uint8_t Train::getProtocol()
{
    return _protocol;
}
